# $Id: changeowner.py,v 1.12 2003/10/05 04:46:44 jrouilla Exp $

# Purpose if the owner changes, send an email to the new owner
# with more info about the ticket including inital email
# request.

# Author: John Rouillard (rouilj@cs.umb.edu)

import re, os, smtplib, socket, time 
import MimeWriter, cStringIO, quopri

from roundup.rfc2822 import encode_header

from roundup import history

# from roundupdb.py
# if available, use the 'email' module, otherwise fallback to 'rfc822'
try :
    from email.Utils import formataddr as straddr
except ImportError :
    # code taken from the email package 2.4.3
    def straddr(pair, specialsre = re.compile(r'[][\()<>@,:;".]'),
            escapesre = re.compile(r'[][\()"]')):
        name, address = pair
        if name:
            quotes = ''
            if specialsre.search(name):
                quotes = '"'
            name = escapesre.sub(r'\\\g<0>', name)
            return '%s%s%s <%s>' % (quotes, name, quotes, address)
        return address

def sendAssignedtoEmail(db, cl, nodeid, oldvalues):
    """Generate an email message with issue context when owner changes

       Send an email composed of some front matter, the initial
       request email, and the last 3 emails (configurable) on the
       issue all in the body of the email. Don't forward any
       attachments from the forwarded messages. This will send email
       even if no message was created when the owner changed.

       FIXME: nosy reactors should know is this module is in use and
              not send email to the new assignedto person relying on
	      this message to act as the nosy message for the
              assignedto person.
    """
    # this is a configuration variable of the number of messages
    # to include
    messagesToInclude = 3

    if __debug__:
        print "changeowner.sendAssignedtoEmail[reactor]: triggered"
        print "sendAssignedtoEmail: nodeid and oldvalues:", nodeid, oldvalues
    
    # if set to true, and there was a message when the issue was
    # created, don't send an assigned email as well. If there was
    # no message typed in (meaning that no nosy email got send)
    # then send the assigned email regardless of the setting of this
    # variable.
    sendNoMessageOnCreate = False

    # If oldvalues is None, then the issue was just created,
    # and the assignedto person hopefully has all the info in
    # the inital nosy message.  However this is not true if no message
    # body was present. So we return here only if it is not a create AND
    # there is no assignedto key, OR the assignedto person has not
    # changed rather then just returning.
    if oldvalues != None:
	if not oldvalues.has_key('assignedto'):
	    # don't care assignedto wasn't changed
	    # FIXME: WHY does this always pass if on a set?
	    # shouldn't old values only have changed values?
	    return

	if oldvalues['assignedto'] == cl.get(nodeid, 'assignedto'):
	    # don't care assignedto wasn't changed
	    return

    if cl.get(nodeid, 'assignedto') == db.getuid():
	# the assignee assigned him/herself basically they took
        # the ticket.
        return

    # either this is a create, or we have changed owner.
    # the action was a set (possibly adding new messages to an existing issue)
    messageidList=[]
    messageidList = cl.get(nodeid, 'messages')

    userid = cl.get(nodeid, 'assignedto')
    # Stop lookup if assignedto is nothing. This
    # will be triggered on initial message create.
    if userid is None: 
        return
    sendto=[]
    sendto.append(db.user.get(userid,"address"))

    last_messages = []
    message_one=None
    if len(messageidList) != 0:
      # locate the id of initial message
      message_one = messageidList[0]

      # get the id's of the last messages. Since the messages are listed in
      # order of arrival, this is just a slice.
      if len(messageidList) > 1:
	  if len(messageidList) < messagesToInclude+1:
	     last_messages = messageidList[1:]
	  else:
	     last_messages = messageidList[-1*messagesToInclude:]

    # if we don't want to send an assignment message on create
    # and prefer to let the create nosy message do the job,
    # set sendNoMessageOnCreate. It will not send a message if the
    # nosy reactors had sent a message. If however no message is
    # found for the ticket, then send an email.
    if sendNoMessageOnCreate and (oldvalues == None) and (message_one != None):
       return

    # get the current property list note. We use the
    # generateCreateNote since we want a current state
    # note not a list of changes.
    note = cl.generateCreateNote(nodeid)

    # Now assemble the components into an email and send to new user
    m = ['']
    messages = cl.db.msg

    # put in roundup's signature
    if db.config.EMAIL_SIGNATURE_POSITION == 'top':
	m.append(cl.email_signature(nodeid, 0))

    # add front matter
    m.append('''
The following issue has been assigned to you. Responding to this email
will send email to all watchers/nosy people on the ticket. To bring
you up to date, this email contains the initial request followed by
the last %d messages on the issue in reverse order, and the values of
the current fields for the issue.

'''%messagesToInclude)

    # add the first message
    if message_one == None:
        m.append('**** No initial or subsequent messages recorded ****')
    else:
        m.append('++++ Initial Message ++++')
        m.append("Date: %s"%messages.get(message_one, 'creation').pretty())
        m.append(messages.get(message_one, 'content'))

	# add subsequent messages
	if len(last_messages) > 0:
	    last_messages.reverse
  	    for message in last_messages:
	        m.append('')
		m.append('++++ Recent Message ++++')
		m.append("Date: %s"%messages.get(message, 'creation').pretty())
		m.append(messages.get(message, 'content'))
	else :
	    m.append('**** No subsequent messages recorded ****')

    # add the field values
    if note:
	m.append(note)

    # put in roundup's signature
    if db.config.EMAIL_SIGNATURE_POSITION == 'bottom':
	m.append(cl.email_signature(nodeid, 0))

    # encode the content as quoted-printable
    content = cStringIO.StringIO('\n'.join(m))
    content_encoded = cStringIO.StringIO()
    quopri.encode(content, content_encoded, 0)
    content_encoded = content_encoded.getvalue()

    # make sure the To line is always the same (for testing mostly)
    sendto.sort()

    # create the message
    message = cStringIO.StringIO()
    writer = MimeWriter.MimeWriter(message)
    writer.addheader('Subject', '[%s%s] ASSIGNED %s'%(cl.classname,
          nodeid, cl.get(nodeid, 'title')))
    writer.addheader('To', ', '.join(sendto))

    from_address = cl.db.config.TRACKER_EMAIL

    # don't use the comment address since new assignee will
    # probably want to talk to watchers for info.
    authid = db.getuid()
    authname = db.user.get(authid, 'realname')
    if not authname:
        authname = db.user.get(authid, 'username')

    # additional bit for after the From: "name"
    from_tag = getattr(cl.db.config, 'EMAIL_FROM_TAG', '')
    if from_tag:
        from_tag = ' ' + from_tag

    writer.addheader('From', straddr((encode_header(authname) + 
            from_tag, from_address)))
    writer.addheader('Reply-To', straddr( 
				    (cl.db.config.TRACKER_NAME,
				     from_address) ) )
    writer.addheader('Date', time.strftime("%a, %d %b %Y %H:%M:%S +0000",
	time.gmtime()))
    writer.addheader('MIME-Version', '1.0')

    # add a uniquely Roundup header to help filtering
    writer.addheader('X-Roundup-Name', db.config.TRACKER_NAME)

    # avoid email loops
    writer.addheader('X-Roundup-Loop', 'hello')

    # overkill since we aren't sending attchments, but...
    writer.addheader('Content-Transfer-Encoding', 'quoted-printable')
    body = writer.startbody('text/plain')
    body.write(content_encoded)

    # now try to send the message
    SENDMAILDEBUG = os.environ.get('SENDMAILDEBUG', '')
    if SENDMAILDEBUG:
	open(SENDMAILDEBUG, 'a').write('FROM: %s\nTO: %s\n%s\n'%(
	    db.config.ADMIN_EMAIL,
	    ', '.join(sendto),message.getvalue()))
    else:
	try:
	    # send the message as admin so bounces are sent there
	    # instead of to roundup
	    smtp = smtplib.SMTP(db.config.MAILHOST)
	    smtp.sendmail(db.config.ADMIN_EMAIL, sendto,
		message.getvalue())
	except socket.error, value:
	    raise MessageSendError, \
		"Couldn't send confirmation email: mailhost %s"%value
	except smtplib.SMTPException, value:
	    raise MessageSendError, \
		"Couldn't send confirmation email: %s"%value

    return

def sendUnAssignedtoEmail(db, cl, nodeid, oldvalues):
    """Generate an email message composed of some front matter,
       and the initial request email telling the old owner that
       they are no longer assigned to this ticket. This will send email
       even if no message was created when the owner changed.
    """
    # FIXME not yet operational

    if __debug__:
        print "changeowner.sendUnassignedtoEmail[reactor]: triggered"
        print "sendUnassignedtoEmail: nodid and oldvalues:", nodeid, oldvalues

    # if oldvalues is None, then the issue was just created.
    if oldvalues != None :
       if not oldvalues.has_key('assignedto'):
	   # don't care assignedto wasn't changed
	   # FIXME: WHY does this always pass if on a set?
	   # shouldn't old values only have changed values?
	   return

       if oldvalues['assignedto'] == cl.get(nodeid, 'assignedto'):
	   # don't care assignedto wasn't changed
	   return

       if oldvalues['assignedto'] == db.getuid():
	   # the old assignee ditched the ticket, so s/he doesn't need
	   # notification
	   return
    else:
        # return immediatly since there is no old assigned to person
        # on a create.
        return
    return

def init(db):
    db.issue.react('set', sendAssignedtoEmail)
    db.issue.react('create', sendAssignedtoEmail)
    db.issue.react('set', sendUnAssignedtoEmail)
    db.issue.react('create', sendUnAssignedtoEmail)
