#$Id: dependson_semantics.py,v 1.9 2003/10/05 04:46:44 jrouilla Exp $

from roundup import date
import time, random

def dependson_reactor(db, cl, nodeid, oldvalues):
    ''' Sets more semantics for dependson relationship.

       If one of the dependees is unresolved, then (2) the
       dependers must be unresolved. Also if the dependees are all
       resolved, and the depender is not open, (3) it should be opened.
       In this case add a message that it is being opened so verynosy
       (technical list) notifications are sent out since assigned to people
       are on the technical list.
    '''
    if __debug__:
	print "dependson_semantics.dependson_reactor[reactor]: triggered"
        print "dependson_reactor: nodeid and oldvalues", nodeid, oldvalues

    if  oldvalues is None or (not oldvalues.has_key('status')):
        # if there is no old status, then we are done.
        return

    resolved = db.status.lookup('resolved')
    newstatus = cl.get(nodeid, 'status')
    oldstatus = oldvalues['status']

    if newstatus != resolved and oldstatus != resolved:
        # again we are done
	return

    # FIXME need to time this with increasing numbers of tickets.
    dependers = cl.find(dependson=nodeid)
    if __debug__:
	print "dependson_reactor: resolve state found"
	print "dependson_reactor: Dependers are: ", dependers

    if dependers == []:
	# nothing to do
	return

    # now check to see if I should open my dependers
    # because I have transitioned from a resolved state (2)
    if oldstatus == resolved:
	for issueid in dependers:
	    openstate = True
	    if cl.get(issueid, 'status') == resolved:
	       if __debug__:
		   print "dependson_reactor: opening issue %s"%issueid
	       cl.set(issueid, status=db.status.lookup('open'))

    # now check to see if I should open my dependers
    # because I have transitioned to a resolved state (3)
    if newstatus == resolved:
	for issueid in dependers:
	    openstate = True
	    for dep in cl.get(issueid, 'dependson'):
		if cl.get(dep, 'status') != resolved and dep != nodeid:
		    if __debug__:
		       print "dependson_reactor: found unresolved dependency %s for parent no open"%dep
		    openstate = False
		    break

	    if openstate:
	       # with nosyreactors, this is probably not needed since adding
	       # the message will open. However I want the functionality
               # in here too so that reopening is not dependedent on
               # nosyreactors.
	       if __debug__:
		   print "dependson_reactor: opening issue %s"%issueid
	       cl.set(issueid, status=db.status.lookup('open'))
	       content = """
issue%s has been resolved. Issues that
depend on this issue are being automatically opened.
"""%(nodeid)
	       if __debug__:
	           print "dependson_reactor: message content is: ", content
               messageid = "<%s.%s.dependson%s@internal>"%(time.time(),
 	                    random.random(), issueid)
	       messagetype = db.msgtype.lookup('comment - to technical')
	       if __debug__:
	           print "dependson_reactor: messagetype is: ", messagetype
	       openmsg = db.msg.create(author=db.getuid(),
			     recipients=[], date=date.Date('.'),
                             summary="Opening issue with dependson relation.",
			     content=content, files=[], messageid=messageid,
			     messagetype=messagetype)
	       if __debug__:
	           print "dependson_reactor: openmsg is: ", openmsg
	       issuemsgs = cl.get(issueid, 'messages')
	       if __debug__:
	           print "dependson_reactor: issuemessages are: ", issuemsgs
	       issuemsgs.append(openmsg)
	       if __debug__:
	           print "dependson_reactor: issuemessages changed to: ", issuemsgs
	       cl.set(issueid, messages=issuemsgs)
    return

def dependson_auditor(db, cl, nodeid, newvalues):
    '''Verify that the status is able to be set correctly

       If an issue (the depender) depends on b, c, d and e (the
       dependees), then (1) the depender must not go into the resolved
       state unless all of the dependees (b, c, d and e) are
       resolved.
    '''
    if __debug__ :
        print "dependson_semantics.dependson_auditor[auditor]: triggered"
        print "dependson_auditor: nodeid and newvalues:", nodeid, newvalues

    if not newvalues.has_key('status'):
        # if there is no new status, then we are done.
        return
    else:
        newstatus = newvalues['status']

    if nodeid == None:
       oldstatus = None
    else:
       oldstatus = cl.get(nodeid, 'status')

    resolved = db.status.lookup('resolved')
    if newstatus != resolved:
        # again we are done
	return

    # ok we have now changed status to resolved
    # see if we are allowed to be resolved. item (1)
    dependees = []
    if newvalues.has_key("dependson"):
	dependees = newvalues["dependson"]
    else:
        if nodeid is not None:
            # an already existing node
	    dependees = cl.get(nodeid, 'dependson')

    if __debug__:
        print nodeid, "dependson_auditor: dependees are: ", dependees

    unresolved = []
    for issueid in dependees:
	if db.issue.get(issueid, 'status') != resolved:
	    unresolved.append(issueid)

    if unresolved != []:
        issues=[]
        for u in unresolved:
            issues.append("<a href=issue%s>issue%s</a>"%(u,u))
        raise ValueError, "Unable to resolve <a href=issue%s>issue%s</a> because dependedon issue(s) "%(nodeid,nodeid) + ', '.join(issues) + " must be resolved first."

    return


def init(db):
    # fire before changes are made
    db.issue.audit('set', dependson_auditor)
    db.issue.audit('create', dependson_auditor)

    db.issue.react('set', dependson_reactor)
    db.issue.react('create', dependson_reactor)


# vim: set filetype=python ts=4 sw=4 et si
#SHA: 8b886dd30aaffe232873d1bcdeec179cdaa10aa2
