#
# Copyright (c) 2001 Bizar Software Pty Ltd (http://www.bizarsoftware.com.au/)
# This module is free software, and you may redistribute it and/or modify
# under the same terms as Python, so long as this copyright message and
# disclaimer are retained in their original form.
#
# IN NO EVENT SHALL BIZAR SOFTWARE PTY LTD BE LIABLE TO ANY PARTY FOR
# DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
# OUT OF THE USE OF THIS CODE, EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# BIZAR SOFTWARE PTY LTD SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
# BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
# 
#$Id: nosy_keyword.py,v 1.2 2003/10/05 04:46:44 jrouilla Exp $

#From: K Schutte <schutte@fel.tno.nl>
#Organization: TNO-FEL
#To: roundup-users@lists.sourceforge.net
#CC: Richard Jones <richardjones@optushome.com.au>
#Subject: Re: [Roundup-users] How do I monitor new issues with topic <foo>?
#
#Richard Jones wrote:
#> 
#> On Mon, 18 Aug 2003 09:50 pm, K Schutte wrote:
#> > How should I set up Roundup such that for any new issue with topic <foo>
#> > I am put on the nosy list?
#> > Does this also work when with an existing issue the topic <foo> is added?
#> > How can this be arranged such that any user (that is, including those
#> > who can't edit Python) can specify on  which topics they will be added
#> > on the nosy list?
#> 
#> You would set up a new reactor that watches for issue create or edit events
#> and checks the topic list for the <foo> topic. You could also have it remove
#> you from the nosy list when the <foo> topic is removed.
#> 
#> Having it be arbitrary would be a small additional change: you'd add a topic
#> mulitlink to the user class so that users can select topics they're
#> interested in. Then in the above reactor, you scan the users to figure out
#> who's interested in what, and modify the nosy list accordingly.
#> 
#> Have a gander at the customisation doc - it has several examples at the
#> bottom, and documentation on how this stuff all works. Don't be afraid to ask
#> for more help though - I'll even knock up an untested example reactor if
#> prodded right :)
#
#See attached a patch containing the diffs for enabling such a capability.
#Apply it to a classic tracker, and you will have the capability to set
#nosy topics for every user.
#

# slightly modified for debugging output -- rouilj

from roundup import roundupdb, hyperdb

def update_kw_nosy(db, cl, nodeid, newvalues):
    '''Update the nosy list for changes to the topics
    '''
    if __debug__:
        print "nosy_keyword.update_kw_nosy[auditor]: triggered"
        print "update_kw_nosy: nodeid and newvalues:", nodeid, newvalues

    # nodeid will be None if this is a new node
    current = {}
    if nodeid is None:
        ok = ('new', 'yes')
    else:
        ok = ('yes',)
        # old node, get the current values from the node if they haven't
        # changed
        if not newvalues.has_key('nosy'):
            nosy = cl.get(nodeid, 'nosy')
            for value in nosy:
                if not current.has_key(value):
                    current[value] = 1

    # if the nosy list changed in this transaction, init from the new value
    if newvalues.has_key('nosy'):
        nosy = newvalues.get('nosy', [])
        for value in nosy:
            if not db.hasnode('user', value):
                continue
            if not current.has_key(value):
                current[value] = 1

    # add users with topic in nosy_keywords to the nosy list
    if newvalues.has_key('topic') and newvalues['topic'] is not None:
        topic_ids = newvalues['topic']
        for topic in topic_ids:
            #print 'doing topic ', topic
            # loop over all users,
            # and assign user to nosy when topic in nosy_keywords
            for user_id in db.user.list():
                #print 'doing user ', user_id
                nosy_kw = db.user.get(user_id, "nosy_keywords")
                #print 'nosy_kw ', nosy_kw
                found = 0
                for kw in nosy_kw:
                    if kw == topic:
                        found = 1
                        #print 'found'
                if found:
                    current[user_id] = 1

    # that's it, save off the new nosy list
    newvalues['nosy'] = current.keys()

def init(db):
    db.issue.audit('create', update_kw_nosy)
    db.issue.audit('set', update_kw_nosy)

# vim: set filetype=python ts=4 sw=4 et si
#SHA: 51f78b7ae574fa97b3e0750fa10dbe06282fa28d
