# Copyright (c) 2002 ekit.com Inc (http://www.ekit-inc.com/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#$Id: statusauditor.py,v 1.8 2003/10/05 04:46:44 jrouilla Exp $

def autoopen(db, cl, nodeid, newvalues):
    ''' If the issue is currently 'new', 'stalled', 'resolved',
        'dead', or 'delete' then set it to 'open'

        Unless the issue is new and the message is from the same
        person who started the issue. Thus multiple "starting"
        messages may be sent without incorrectly opening the ticket.
    '''

    allowmultiplecreate = True

    if __debug__:
       print "statusauditor.autoopen[auditor]: triggered"
       print "autoopen: nodeid and newvalues:", nodeid, newvalues

    # don't fire if there's no new message (ie. chat)
    if not newvalues.has_key('messages'):
        return
    if newvalues['messages'] == cl.get(nodeid, 'messages'):
        return

    # get the open state ID
    try:
        chatting_id = db.status.lookup('open')
    except KeyError:
        # no open state, ignore all this stuff
        return

    # get the current value
    current_status = cl.get(nodeid, 'status')

    if __debug__:
       print "autoopen: current_status id is: %s"%current_status

    # see if there's an explicit change in this transaction
    if newvalues.has_key('status'):
        # yep, skip
        return

    # get the value for new.
    new = db.status.lookup('new')

    if current_status == new and allowmultiplecreate:
        if __debug__:
	    print "autoopen: new status detected"
        messages = cl.get(nodeid, 'messages')
        if messages:
            if __debug__:
	        print "autoopen: messages detected"
	    # find author of first message if same, then don't open.
	    firstmessageid = messages[0]
	    author = db.msg.get(firstmessageid, 'author')
	    if __debug__:
	        print "autoopen: first message id %s, Author id %s, uid %s"%(firstmessageid, author, db.getuid())
	    # if the author of the current message is the author of
            # the first message don't open. This is only another
            # starter message.
	    if author == db.getuid():
	        return

    # determine the id of 'new', 'stalled', 'resolved',
    # 'dead' or 'delete'
    fromstates = []
    for state in 'new stalled resolved dead delete'.split():
        try:
            fromstates.append(db.status.lookup(state))
        except KeyError:
            pass

    # ok, there's no explicit change, so check if we are in a state that
    # should be changed
    if current_status in fromstates:
        # yep, we're now chatting
        newvalues['status'] = chatting_id


def presetnew(db, cl, nodeid, newvalues):
    ''' Make sure the status is set on new issues
    '''
    if __debug__:
        print "statusauditor.presetnew[auditor] triggered"

    if newvalues.has_key('status') and newvalues['status']:
        return

    # ok, do it
    newvalues['status'] = db.status.lookup('new')


def init(db):
    # fire before changes are made
    db.issue.audit('set', autoopen)
    db.issue.audit('create', presetnew)

# vim: set filetype=python ts=4 sw=4 et si
#SHA: 8b886dd30aaffe232873d1bcdeec179cdaa10aa2
