#
# Copyright (c) 2001 Bizar Software Pty Ltd (http://www.bizarsoftware.com.au/)
# This module is free software, and you may redistribute it and/or modify
# under the same terms as Python, so long as this copyright message and
# disclaimer are retained in their original form.
#
# IN NO EVENT SHALL BIZAR SOFTWARE PTY LTD BE LIABLE TO ANY PARTY FOR
# DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
# OUT OF THE USE OF THIS CODE, EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# BIZAR SOFTWARE PTY LTD SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
# BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
# 
#$Id: verynosy.py,v 1.3 2003/10/05 04:46:44 jrouilla Exp $

from roundup import roundupdb, hyperdb

def verynosyreaction(db, cl, nodeid, oldvalues):
    ''' A standard detector is provided that watches for additions to the
        "messages" property.
        
        When a new message is added, the detector sends it to all the users on
        the "nosy" list for the issue that are not already on the "recipients"
        list of the message.
        
        Those users are then appended to the "recipients" property on the
        message, so multiple copies of a message are never sent to the same
        user.
        
        The journal recorded by the hyperdatabase on the "recipients" property
        then provides a log of when the message was sent to whom. 
    '''
    # send a copy of all new messages to the nosy list
    for msgid in determineNewMessages(cl, nodeid, oldvalues):
        try:
	    # get a message ID and type
	    messages = cl.db.msg
	    ccString = messages.get(msgid, 'cc')
	    msgTypeID = messages.get(msgid, "messagetype")
	    msgType = cl.db.msgtype.get(msgTypeID, "name")
	    
	    # if message type is not reply, don't send it in this
            # reactor
	    if not msgType.startswith('comment -'):
	       continue

	    # check every message for a cc list and find the addresses
            # on the cc list creating them if they don't exist. Then
	    # pass the list of user id's to cl.nosymessage
	    if (ccString != None) and (len(ccString) != 0):
                ccList = ccString.split(',')
		ccUidList=[]
		for ccAddress in ccList:
		    ccUidList.append(
			uidFromAddress(cl.db, (ccAddress, ccAddress))
			)
		cl.nosymessage(nodeid, msgid, oldvalues,
			       whichnosy='verynosy',
			       from_address=cl.db.config.COMMENT_TRACKER_EMAIL,
			       cc=ccUidList)
            else:
		cl.nosymessage(nodeid, msgid, oldvalues,
			       whichnosy='verynosy',
			       from_address=cl.db.config.COMMENT_TRACKER_EMAIL)
        except roundupdb.MessageSendError, message:
            raise roundupdb.DetectorError, message

def determineNewMessages(cl, nodeid, oldvalues):
    ''' Figure a list of the messages that are being added to the given
        node in this transaction.
    '''
    messages = []
    if oldvalues is None:
        # the action was a create, so use all the messages in the create
        messages = cl.get(nodeid, 'messages')
    elif oldvalues.has_key('messages'):
        # the action was a set (so adding new messages to an existing issue)
        m = {}
        for msgid in oldvalues['messages']:
            m[msgid] = 1
        messages = []
        # figure which of the messages now on the issue weren't there before
        for msgid in cl.get(nodeid, 'messages'):
            if not m.has_key(msgid):
                messages.append(msgid)
    return messages

def updateverynosy(db, cl, nodeid, newvalues):
    '''Update the nosy list for changes to the assignedto
    '''
    if __debug__:
        print "verynosy.updateverynosy[auditor]: triggered"
        print "updateverynosy: nodeid and newvalues:", nodeid, newvalues

    # nodeid will be None if this is a new node
    current = {}
    if nodeid is None:
        ok = ('new', 'yes')
    else:
        ok = ('yes',)
        # old node, get the current values from the node if they haven't
        # changed
        if not newvalues.has_key('verynosy'):
            nosy = cl.get(nodeid, 'verynosy')
            for value in nosy:
                if not current.has_key(value):
                    current[value] = 1

    # if the nosy list changed in this transaction, init from the new value
    if newvalues.has_key('verynosy'):
        nosy = newvalues.get('verynosy', [])
        for value in nosy:
            if not db.hasnode('user', value):
                continue
            if not current.has_key(value):
                current[value] = 1

    # add assignedto(s) to the nosy list
    if newvalues.has_key('assignedto') and newvalues['assignedto'] is not None:
        propdef = cl.getprops()
        if isinstance(propdef['assignedto'], hyperdb.Link):
            assignedto_ids = [newvalues['assignedto']]
        elif isinstance(propdef['assignedto'], hyperdb.Multilink):
            assignedto_ids = newvalues['assignedto']
        for assignedto_id in assignedto_ids:
            if not current.has_key(assignedto_id):
                current[assignedto_id] = 1

    # that's it, save off the new nosy list
    newvalues['verynosy'] = current.keys()

# Stolen from mailgw.py
def extractUserFromList(userClass, users):
    '''Given a list of users, try to extract the first non-anonymous user
       and return that user, otherwise return None
    '''
    if len(users) > 1:
        for user in users:
            # make sure we don't match the anonymous or admin user
            if userClass.get(user, 'username') in ('admin', 'anonymous'):
                continue
            # first valid match will do
            return user
        # well, I guess we have no choice
        return user[0]
    elif users:
        return users[0]
    return None

def uidFromAddress(db, address, create=1, **user_props):
    ''' address is from the rfc822 module, and therefore is (name, addr)

        user is created if they don't exist in the db already
    '''
    (realname, address) = address

    # try a straight match of the address
    user = extractUserFromList(db.user, db.user.stringFind(address=address))
    if user is not None: return user

    # try the user alternate addresses if possible
    props = db.user.getprops()
    if props.has_key('alternate_addresses'):
        users = db.user.filter(None, {'alternate_addresses': address})
        user = extractUserFromList(db.user, users)
        if user is not None: return user

    # try to match the username to the address (for local
    # submissions where the address is empty)
    user = extractUserFromList(db.user, db.user.stringFind(username=address))

    # couldn't match address or username, so create a new user
    if create:
        return db.user.create(username=address, address=address,
            realname=realname, roles=db.config.NEW_EMAIL_USER_ROLES, **user_props)
    else:
        return 0

def init(db):
    db.issue.react('create', verynosyreaction)
    db.issue.react('set', verynosyreaction)
    db.issue.audit('create', updateverynosy)
    db.issue.audit('set', updateverynosy)

# vim: set filetype=python ts=4 sw=4 et si
#SHA: 509a13c8501bbdf8d171ddab4e91c4ff0b9da957
