#
# Copyright (c) 2001 Bizar Software Pty Ltd (http://www.bizarsoftware.com.au/)
# This module is free software, and you may redistribute it and/or modify
# under the same terms as Python, so long as this copyright message and
# disclaimer are retained in their original form.
#
# IN NO EVENT SHALL BIZAR SOFTWARE PTY LTD BE LIABLE TO ANY PARTY FOR
# DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
# OUT OF THE USE OF THIS CODE, EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# BIZAR SOFTWARE PTY LTD SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
# BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
# 
# $Id: interfaces.py,v 1.14 2003/05/18 23:05:09 jrouilla Exp $

from roundup import mailgw 
from roundup.cgi import client
from roundup.date import Interval
from cgi import MiniFieldStorage
import urllib

class Client(client.Client): 
    ''' derives basic CGI implementation from the standard module, 
        with any specific extensions 
    ''' 

    actions = client.Client.actions + (
       ('show', 'showAction'),
    )

    def showAction(self):
        ''' shows the issue when a number is typed into the form on the page

            The proper action on an invalid, blank or non-existant issue
            number is to redisplay the requesting page with an error banner
            at the top of the page.

            FIXME doesn't work properly on index pages since it
            doesn't append the query part of the index url.
            Also doesn't handle the case where the issue number
            doesn't exist.
        '''

        t=None
        n=None
        if self.form.has_key('@type'):
	    t = self.form['@type'].value
            if self.form.has_key('@number'):
                n = self.form['@number'].value
            if (t != None) and (n != None):
                url = '%s%s%s'%(self.db.config.TRACKER_WEB,t,n)
                raise client.Redirect, url
            else:
	        if n == None:
		    if self.nodeid:
	                raise client.Redirect, '%s%s%s?@error_message=%s'%(self.base, self.classname, self.nodeid,  urllib.quote('You must enter an issue number'))
		    else:
                        raise client.Redirect, '%s%s?@error_message=%s'%(self.base, self.classname,  urllib.quote('You must enter an issue number'))

        else:
            raise ValueError, 'Missing @type parameter check "page" html template'

class TemplatingUtils:
    ''' Methods implemented on this class will be available to HTML templates
        through the 'utils' variable.
    '''
    def totalTimeSpent(self, times):
        ''' Call me with a list of timelog items (which have an Interval
            "period" property)
        '''
        total = Interval('0:00')
        for time in times:
            total += time.period._value
        return total

    def generateQuotedReply(self, msgid):
        ''' Called with a message id it generates quoted text
        '''
	msgdb = self.client.db.msg
	userdb = self.client.db.user
	authorid = msgdb.get(msgid, 'author')
	if __debug__:
	    print "message authorid: %s username: %s"%(authorid, userdb.get(authorid, 'username'))
        realname = userdb.get(authorid, 'realname')
	username = userdb.get(authorid, 'username')
	if realname is None:
	    if username is None :
                name = "unknown author"
            else:
	        name = username.replace("@", " at ")
        else:
	    name = "%s (user%s)"%(realname, authorid)

	message = "In message msg%s on %s, %s said:\n>"%(msgid,
                    msgdb.get(msgid, 'date'), name) +  \
                    msgdb.get(msgid, 'content').replace("\n","\n>")
	if __debug__:
	    print "message is: ", message
	return message

    def AddToCurrentUrlQuery(self, request, context, queryAdd):
	''' create a URL and add the queryAdd to the end of it in the
	    query.
	'''
	if context is not None and context.id:
            url=request.base+request.classname+context.id
	else:
	    if request is not None and request.classname:
	         url=request.base+request.classname
            else:
 	        url=request.base+'home'

	if request.env.has_key('QUERY_STRING'):
           url=url + '?%s'%request.env['QUERY_STRING']
	   url=url+'&%s'%queryAdd
        else:
	   url=url+'?%s'%queryAdd
        return url

    def GenerateHiddenFieldsForForm(self, request):
	''' create a series of <hidden inputs> that will allow a new form
            to keep the current query arguments. This is used to
	    implement user settable refresh rates.
 	'''
        elements=''
	if request.env.has_key('QUERY_STRING'):
           queryElements = []
	   queryElements=request.env['QUERY_STRING'].split('&')
	   for e in queryElements:
               s=e.split('=', 1)
	       if len(s) == 2:
     	           elements=elements + '\n <INPUT TYPE=HIDDEN NAME="%s" VALUE="%s">'%(s[0], s[1])
        return elements

class MailGW(mailgw.MailGW): 
    ''' derives basic mail gateway implementation from the standard module, 
        with any specific extensions 
    ''' 
    pass

# vim: set filetype=python ts=4 sw=4 et si
#SHA: 74cd55606c7db17a1f9c20ce42534917a478ddb8
